
/*	Copyright (C) 2005, 2006 by Andy Grebe a.k.a. tecknophreak(tecknophreak@gmail.com)
 *
 *	This file is part of TouchMe Games.
 *
 *	TouchMe Games is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	(at your option) any later version.
 *
 *	TouchMe Games is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with TouchMe Games; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

// support functions for VIDEO_POKER

#include <videopoker.h>
#include <algorithm>
#include <sys/time.h>
#include <sstream>

VIDEO_POKER::VIDEO_POKER() : gameName("videopoker"), toggleOccuring(false),
 		deckLocation("/usr/local/share/touchmegames/images/cards/"),
		deckBack("/usr/local/share/touchmegames/images/cards/backs/back-2.gif") {
	color.red = 0xA000;
	color.green = 0xA000;
	color.blue = 0xA000;
}

std::string VIDEO_POKER::name() const {
	return gameName;
}

std::string VIDEO_POKER::imageName() const {
	return "videopoker.gif";
}

GdkColor *VIDEO_POKER::backgroundColor() {
	return &color;
}

void VIDEO_POKER::shuffle(int *deck) {
	timeval tv;
	struct timezone tz;
	int deckcopy[52];
	int lh, rh(51), middle;
	int pos(51);
	
	tz.tz_minuteswest = 0;
	tz.tz_dsttime = 0;

	gettimeofday(&tv, &tz);
	srandom(tv.tv_usec);
	
	middle = random() % 52;
	lh = middle;
	
	while (rh != middle && lh != -1) {
		deckcopy[pos--] = deck[lh--];
		deckcopy[pos--] = deck[rh--];
	}
	while (rh != middle) {
		deckcopy[pos--] = deck[rh--];
	}
	while (lh != -1) {
		deckcopy[pos--] = deck[lh--];
	}
	// now cut the deck
	
	middle = random() % 52;
	lh = middle;
	pos = 51;
	while (lh >= 0) {
		deck[pos--] = deckcopy[lh--];
	}
	rh = 51;
	while (rh > middle) {
		deck[pos--] = deckcopy[rh--];
	}
	
}

void VIDEO_POKER::startHand() {
	// shuffle
	for (int i = 0; i < 10; ++i) {
		shuffle(cardDeck);
	}
	cardLocation = 0;
	
	
	gameState = SELECT_BET;
}

bool VIDEO_POKER::isFlush() {
	int suit(cardNumber[0] % 4);
	for (int i = 1; i < 5; ++i) {
		if (cardNumber[i] % 4 != suit) {
			return false;
		}
	}
	return true;
}

bool VIDEO_POKER::isStraight() {
	int cards[5];
	bool firstTest(true);
	for (int i = 0; i < 5; ++i) {
		cards[i] = cardNumber[i] / 4;
	}
	for (int i = 0; i < 4; ++i) {
		if (cards[i] != cards[i + 1] + 1) {
			firstTest = false;
		}
	}
	if (firstTest) {
		return true;
	}
	if (cards[0] == 0) {// ace
		cards[0] = 12;
	}
	
	for (int i = 0; i < 4; ++i) {
		if (cards[i] != cards[i + 1] - 1) {
			return false;
		}
	}
	return true;
}

int VIDEO_POKER::pairs() {
	int card;
	int numPairs(0);
	for (int i = 0; i < 5; ++i) {
		int numOfCard(1);
		int idx;
		card = cardNumber[i] / 4;
		for (idx = i + 1; idx < 5; ++idx) {
			if (cardNumber[idx] / 4 == card) {
				++numOfCard;
			}
		}
		if (numOfCard == 2) {
			++numPairs;
		}
		i += numOfCard - 1;
	}
	return numPairs;
}

int VIDEO_POKER::trips() {
	int card;
	int numTrips(0);
	for (int i = 0; i < 5; ++i) {
		int numOfCard(1);
		int idx;
		card = cardNumber[i] / 4;
		for (idx = i + 1; idx < 5; ++idx) {
			if (cardNumber[idx] / 4 == card) {
				++numOfCard;
			}
		}
		if (numOfCard == 3) {
			++numTrips;
		}
		i += numOfCard - 1;
	}
	return numTrips;
}

int VIDEO_POKER::getHand() {
	std::sort(cardNumber, &cardNumber[5], std::greater<int>());
	if (std::count(cardNumber, &cardNumber[5], cardNumber[2]) == 4) {
		return FOUR_OF_A_KIND;
	}
	if (isStraight()) {
		if (isFlush()) {
			if (cardNumber[4] < 4) {
				return ROYAL_FLUSH;
			}
			return STRAIGHT_FLUSH;
		}
		return STRAIGHT;
	}
	if (isFlush()) {
		return FLUSH;
	}
	if (trips()) {
		if (pairs()) {
			return FULL_HOUSE;
		}
		return TRIPS;
	}
	if (pairs() == 2) {
		return TWO_PAIR;
	}
	if (pairs()) {
		return PAIR;
	}
	return HIGH_CARD;
}


int VIDEO_POKER::gameType() const {
	return CARD_GAME;
}

GtkWidget *VIDEO_POKER::getHelp() {
    GtkWidget *vbox;
    GtkWidget *label;
    GdkColor color;
    std::stringstream sstr;
    
    color.red = 0xFFFF;
    color.green = 0xFFFF;
    color.blue = 0xFFFF;
    
    vbox = gtk_vbox_new(FALSE, 0);
    
    sstr << "videopoker is the same as any other video poker\n"
            << "game, except you have 20 rounds to get a high score and\n"
            << "ANY pair count as a pair, not just Jacks and higher.\n";
    
    
    label = gtk_label_new(sstr.str().c_str());
    gtk_widget_modify_fg(label, (GtkStateType)0, &color);
    gtk_box_pack_start((GtkBox *)vbox, label, FALSE, FALSE, 10);
    return vbox;
}
